# ClassProp

ClassProp allows you to set properties of a class which can be inherited or
overridden by subclasses. For example, in the following code, the `Base` class
is created and includes `ClassProp`. Then it defines an inheritable property
called `fco`. Finally, it sets the value of that property to *base*.

```ruby
class Base
    include ClassProp
    define_class_prop 'independent'
    self.independent = 'base'
end
```

The `fco` property is now available from `Base`.

```ruby
puts Base.independent # => 'base'
```

Another class can inherit from `Base`.

```ruby
class X1 < Base
end
```

If `X1` does not define its own `fco` property, then when that property is
retrieved from `X1` it returns `Base.fco`.

```ruby
puts X1.independent # => 'base'
```

`X1` can set its own value for `fco`.

```ruby
class X1
    self.independent = 'x1'
end
```

That gives us this output.

```ruby
puts X1.independent # => 'x1'
```

To delete a class property (so that the property is again inherited from the
superclass) use `delete_class_prop`.

```ruby
class X1
    delete_class_prop 'independent'
end

puts X1.independent # => 'base'
```

You can set a property so that it must be defined by subclasses. To do so, set
the property to `ClassProp::MustDefine`. If an inheriting class does not define
that property then an error is raised when the property is retrieved.

```ruby
class Base
    include ClassProp
    define_class_prop 'independent'
    self.independent = ClassProp::MustDefine
end

class X1 < Base
end

X1.independent # raises exception 'must-define-class-property: independent'
```

If you want to normalize or check a property, use the `before_set` option.

```ruby
class Base
    include ClassProp
    
    # upper
    define_class_prop(
        'upper',
        'before_set' => proc do |val|
            val.to_s.upcase
        end
    );
end

Base.upper = 'whatever'
puts Base.upper #=> WHATEVER
```


## Install

```
gem install classprop
```

## Author

Mike O'Sullivan
mike@idocs.com

## History

| version | date         | notes                                                 |
|---------|--------------|-------------------------------------------------------|
| 1.0     | Feb 24, 2020 | Initial upload.                                       |
| 1.1     | Feb 24, 2020 | Reworked interface.                                   |
| 1.2     | Feb 24, 2020 | Fixed minor problem with exception id.                |
| 1.3     | Feb 26, 2020 | Added options to define_class_prop: init and inherit. |
| 1.3     | Mar 13, 2020 | Added option to define_class_prop: before_set.        |